/**
 * \file
 * \brief Defines mcmc order search
 */

#ifndef BNL_ORDER_SEARCH
#define BNL_ORDER_SEARCH


#include <bnl/random/algorithm/random_swap.hpp>
#include <bnl/score/log_score.hpp>
#include <bnl/score/local_log_bdeu_score.hpp>
#include <bnl/random/randgen.hpp>

#include <bnl/smile_ext/dataset_match.hpp>
#include <bnl/smile_ext/idarray_range.hpp>
#include <bnl/smile_ext/intarray_range.hpp>

#include <bnl/refcounted_dataset.hpp>

#include <bnl/utility/odometer.hpp>
#include <bnl/utility/error.hpp>

#include <boost/lexical_cast.hpp>
#include <algorithm>
#include <string>
#include <fstream>
#include <iterator>
#include <iostream>
#include <utility>
#include <limits>
#include <iomanip>
#include <vector>

namespace bnl
{
/**
 * \brief Input order search parameters allows users to input the maximum iterations and maximum run time allowed for the order search to run
 */
	unsigned input_order_search_iterations();
	unsigned input_order_search_time();
/**
 * \brief MCMC order search using random swap
 * \param order The total order to test on
 * \param ds Reference-counted dataset where variable states are nonnegative ints
 * \param max_time The maximum time allowed for the search to run
 */

	void mcmc_order_search(std::vector<int> order, bnl::refcounted_dataset ds, unsigned max_iterations, unsigned max_time);


} // namespace bnl


#endif //BNL_ORDER_SEARCH
