/**
 * \file
 * \ingroup random
 * \brief Defines the random swap algorithm using a global RNG.
 */

#ifndef BNL_RANDOM_ALGORITHM_RANDOM_SWAP_HPP
#define BNL_RANDOM_ALGORITHM_RANDOM_SWAP_HPP

#include <bnl/random/randgen.hpp>

#include <stdlib.h>  

#include <algorithm>
#include <cassert>
#include <cstddef>
#include <iterator>
#include <utility>
#include <string>
#include <fstream>
#include <iterator>
#include <iostream>
#include <limits>
#include <iomanip>
#include <vector>

namespace bnl
{

/**
 * \brief Random Swap Order randomly swaps two elements in the range [first,last).
 * \param first vector<int> the initial position of a sequence.
 * \param last vector<int> the final position of the sequence.
 * \return A pair containing iterators to the swapped values. If two unique
 *         elements cannot be chosen, both iterators are set to the initial
 *         position. Otherwise, the first value of the pair should precede
 *         the second value in the resulting range.
 */

void random_swap_order(std::vector<int>& od, std::vector<int>& swapped_od);


} // namespace bnl

#endif // BNL_RANDOM_ALGORITHM_RANDOM_SWAP_HPP
