/**
 * \file
 * \brief Defines functions dealing with dataset-network matches.
 */

#ifndef BNL_SMILE_EXT_DATASET_MATCH_HPP
#define BNL_SMILE_EXT_DATASET_MATCH_HPP

#include <smile/smilearn.h>
#include <vector>

namespace bnl
{
namespace smile
{

/**
 * \brief Comparison function for nodes inside dataset matches.
 * \param m1,m2 The two dataset matches to compare.
 * \return true if the first argument's node is less than the second's node.
 * \ingroup smile_ext
 */
bool compare_match_nodes(const ::DSL_datasetMatch& m1, const ::DSL_datasetMatch& m2);

/**
 * \brief Creates a vector that acts like a map from node handles to variables.
 *
 * An index of the vector is a network node handle while the value contained
 * inside an index is a dataset variable. Not all indices may be valid entries,
 * so a user needs to make sure which indices are being accessed.
 *
 * The returned data-structure works well when the node handles are densely
 * packed inside an interval starting from 0 onward. This is typically the case
 * when the dataset is created from the network, or vice-versa.
 *
 * \param matches A vector of structs representing the dataset-network matches.
 * \return A vector that allows random-access of variable from node handle.
 * \ingroup smile_ext
 */
::std::vector<int> make_vector_map(const ::std::vector< ::DSL_datasetMatch>& matches);

} // namespace smile
} // namespace bnl

#endif // BNL_SMILE_EXT_DATASET_MATCH_HPP
