/**
 * \file
 * \brief Boost.Range capabilities to SMILE id array.
 */

#ifndef BNL_SMILE_EXT_IDARRAY_RANGE_HPP
#define BNL_SMILE_EXT_IDARRAY_RANGE_HPP

#include <smile/idarray.h>
#include <boost/range.hpp>

/// \addtogroup smile_ext
///@{

namespace boost
{

/**
 * \brief Specialized metafunction for SMILE int-array's mutable iterator type.
 */
template<>
struct range_mutable_iterator<DSL_idArray>
{
    typedef char** type; ///< The mutable iterator type for DSL_idArray
};

/**
 * \brief Specialized metafunction for SMILE int-array's const iterator type.
 */
template<>
struct range_const_iterator<DSL_idArray>
{
    typedef const char** type; ///< The const iterator type for DSL_idArray
};

} // namespace boost

/// \name Functions for ADL on DSL_idArray
///@{
inline char** range_begin(DSL_idArray& a)
{
    return a.Items();
}

inline const char** range_begin(const DSL_idArray& a)
{
    return const_cast<const char**>(const_cast<DSL_idArray&>(a).Items());
}

inline char** range_end(DSL_idArray& a)
{
    return a.Items() + a.NumItems();
}

inline const char** range_end(const DSL_idArray& a)
{
    return const_cast<const char**>(const_cast<DSL_idArray&>(a).Items() + a.NumItems());
}
///@}

///@}

#endif // BNL_SMILE_EXT_IDARRAY_RANGE_HPP
