/**
 * \file
 * \brief Boost.Range capabilities to SMILE integer array.
 */

#ifndef BNL_SMILE_EXT_INTARRAY_RANGE_HPP
#define BNL_SMILE_EXT_INTARRAY_RANGE_HPP

#include <smile/intarray.h>
#include <boost/range.hpp>

/// \addtogroup smile_ext
///@{

namespace boost
{

/**
 * \brief Specialized metafunction for SMILE id-array's mutable iterator type
 */
template<>
struct range_mutable_iterator<DSL_intArray>
{
    typedef int* type; ///< The mutable iterator type for DSL_intArray
};

/**
 * \brief Specialized metafunction for SMILE id-array's const iterator type
 */
template<>
struct range_const_iterator<DSL_intArray>
{
    typedef const int* type; ///< The const iterator type for DSL_intArray
};

} // namespace boost

/// \name Functions for ADL on DSL_intArray
///@{
inline int* range_begin(DSL_intArray& a)
{
    return a.Items();
}

inline const int* range_begin(const DSL_intArray& a)
{
    return a.Items();
}

inline int* range_end(DSL_intArray& a)
{
    return a.Items() + a.NumItems();
}

inline const int* range_end(const DSL_intArray& a)
{
    return a.Items() + a.NumItems();
}
///@}

///@}

#endif // BNL_SMILE_EXT_INTARRAY_RANGE_HPP
