/**
 * \file
 * \brief Defines exceptions that can occur when using the library.
 */

#ifndef BNL_UTILITY_ERROR_HPP
#define BNL_UTILITY_ERROR_HPP

#include <smile/smile.h>
#include <stdexcept>
#include <string>

namespace bnl
{

/// \addtogroup utility
///@{

/**
 * \brief An exception object thrown due to input/output errors.
 */
class io_error : public ::std::runtime_error
{
public:
/// \name (constructor)
//@{
    explicit io_error(const ::std::string& msg);
    explicit io_error(const char* msg);
//@}
};

/**
 * \brief An exception object thrown due to any SMILE error code.
 *
 * If there is an error message generated by the SMILE API, then that message
 * will be the exception's what() message. Otherwise, the input message is used.
 */
class smile_error : public ::std::runtime_error
{
public:
/// \name (constructor)
//@{
    smile_error(::DSL_errorStringHandler& error_handler, const ::std::string& msg);
    smile_error(::DSL_errorStringHandler& error_handler, const char* msg);
//@}
};

/**
 * \brief An exception object thrown due to a mismatch.
 *
 * The mismatch is based on the information inside the data structures, i.e.
 * between any two of the following: the Bayesian network, the reference-counted
 * dataset, and the total order.
 */
class match_error : public ::std::runtime_error
{
public:
/// \name (constructor)
//@{
    explicit match_error(const ::std::string& msg);
    explicit match_error(const char* msg);
//@}
};

///@}

} // namespace bnl

#endif // BNL_UTILITY_ERROR_HPP
