/**
 * \file
 * \brief Defines functions to deal with Graphviz files.
 */

#ifndef BNL_GRAPHVIZ_HPP
#define BNL_GRAPHVIZ_HPP
#include <PrintXDSL.hpp>
#include <bnl/algorithm/join.hpp>
#include <bnl/smile_ext/idarray_range.hpp>
#include <bnl/utility/error.hpp>
#include <smile/smile.h>

#include <cstdio>
#include <cstdlib>
#include <fstream>
#include <iomanip>
#include <ios>
#include <iostream>
#include <limits>
#include <sstream>
#include <string>
#include <vector>

namespace bnl
{
/**
 * \brief Convert a GraphViz DOT graph into a Bayesian network.
 * \param in An input stream object for the DOT graph.
 * \param bn The corresponding Bayesian network.
 * \throw io_error if any issue reading DOT file or writing SMILE network.
 */
void read_graphviz(::std::istream& in, bayesian_network& bn);

/**
 * \brief Convert a GraphViz DOT graph into a Bayesian network.
 * \param in An input stream object for the DOT graph.
 * \return The corresponding Bayesian network.
 */
bayesian_network read_graphviz(::std::istream& in);

///@}

} // namespace bnl

#endif // BNL_GRAPHVIZ_HPP
